@@ -3,30 +3,32 @@
 
 extern crate test;
 
-use dashmap::DashMap;
-use rand::Rng;
-use rayon::iter::{IntoParallelRefIterator, ParallelIterator};
-use solana_runtime::{
-    accounts::{create_test_accounts, AccountAddressFilter, Accounts},
-    accounts_db::AccountShrinkThreshold,
-    accounts_index::AccountSecondaryIndexes,
-    ancestors::Ancestors,
-    bank::*,
+use {
+    dashmap::DashMap,
+    rand::Rng,
+    rayon::iter::{IntoParallelRefIterator, ParallelIterator},
+    solana_runtime::{
+        accounts::{create_test_accounts, AccountAddressFilter, Accounts},
+        accounts_db::AccountShrinkThreshold,
+        accounts_index::AccountSecondaryIndexes,
+        ancestors::Ancestors,
+        bank::*,
+    },
+    safecoin_sdk::{
+        account::{AccountSharedData, ReadableAccount},
+        genesis_config::{create_genesis_config, ClusterType},
+        hash::Hash,
+        lamports::LamportsError,
+        pubkey::Pubkey,
+    },
+    std::{
+        collections::{HashMap, HashSet},
+        path::PathBuf,
+        sync::{Arc, RwLock},
+        thread::Builder,
+    },
+    test::Bencher,
 };
-use safecoin_sdk::{
-    account::{AccountSharedData, ReadableAccount},
-    genesis_config::{create_genesis_config, ClusterType},
-    hash::Hash,
-    lamports::LamportsError,
-    pubkey::Pubkey,
-};
-use std::{
-    collections::{HashMap, HashSet},
-    path::PathBuf,
-    sync::{Arc, RwLock},
-    thread::Builder,
-};
-use test::Bencher;
 
 fn deposit_many(bank: &Bank, pubkeys: &mut Vec<Pubkey>, num: usize) -> Result<(), LamportsError> {
     for t in 0..num {
@@ -62,6 +64,7 @@ fn test_accounts_create(bencher: &mut Bencher) {
         false,
         AccountShrinkThreshold::default(),
         false,
+        None,
     );
     bencher.iter(|| {
         let mut pubkeys: Vec<Pubkey> = vec![];
@@ -83,6 +86,7 @@ fn test_accounts_squash(bencher: &mut Bencher) {
         false,
         AccountShrinkThreshold::default(),
         false,
+        None,
     ));
     let mut pubkeys: Vec<Pubkey> = vec![];
     deposit_many(&prev_bank, &mut pubkeys, 250_000).unwrap();
@@ -109,6 +113,7 @@ fn test_accounts_hash_bank_hash(bencher: &mut Bencher) {
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     );
     let mut pubkeys: Vec<Pubkey> = vec![];
     let num_accounts = 60_000;
@@ -136,6 +141,7 @@ fn test_update_accounts_hash(bencher: &mut Bencher) {
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     );
     let mut pubkeys: Vec<Pubkey> = vec![];
     create_test_accounts(&accounts, &mut pubkeys, 50_000, 0);
@@ -154,6 +160,7 @@ fn test_accounts_delta_hash(bencher: &mut Bencher) {
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     );
     let mut pubkeys: Vec<Pubkey> = vec![];
     create_test_accounts(&accounts, &mut pubkeys, 100_000, 0);
@@ -171,6 +178,7 @@ fn bench_delete_dependencies(bencher: &mut Bencher) {
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     );
     let mut old_pubkey = Pubkey::default();
     let zero_account = AccountSharedData::new(0, 0, AccountSharedData::default().owner());
@@ -205,6 +213,7 @@ fn store_accounts_with_possible_contention<F: 'static>(
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     ));
     let num_keys = 1000;
     let slot = 0;
@@ -341,6 +350,7 @@ fn setup_bench_dashmap_iter() -> (Arc<Accounts>, DashMap<Pubkey, (AccountSharedD
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     ));
 
     let dashmap = DashMap::new();
@@ -396,6 +406,7 @@ fn bench_load_largest_accounts(b: &mut Bencher) {
         AccountSecondaryIndexes::default(),
         false,
         AccountShrinkThreshold::default(),
+        None,
     );
     let mut rng = rand::thread_rng();
     for _ in 0..10_000 {
