@@ -1,19 +1,20 @@
 #![feature(test)]
 extern crate test;
 
-use rand::{thread_rng, Rng};
-use solana_runtime::append_vec::{
-    test_utils::{create_test_account, get_append_vec_path},
-    AppendVec,
+use {
+    rand::{thread_rng, Rng},
+    solana_runtime::append_vec::{
+        test_utils::{create_test_account, get_append_vec_path},
+        AppendVec,
+    },
+    safecoin_sdk::{account::ReadableAccount, hash::Hash},
+    std::{
+        sync::{Arc, Mutex},
+        thread::{sleep, spawn},
+        time::Duration,
+    },
+    test::Bencher,
 };
-use safecoin_sdk::{account::ReadableAccount, hash::Hash};
-use std::{
-    sync::{Arc, Mutex},
-    thread::sleep,
-    thread::spawn,
-    time::Duration,
-};
-use test::Bencher;
 
 #[bench]
 fn append_vec_append(bencher: &mut Bencher) {
