@@ -1,11 +1,13 @@
-use crate::{
-    bank::{Bank, TransactionResults},
-    genesis_utils::{self, GenesisConfigInfo, ValidatorVoteKeypairs},
-    hashed_transaction::HashedTransaction,
-    vote_sender_types::ReplayVoteSender,
+use {
+    crate::{
+        bank::{Bank, TransactionResults},
+        genesis_utils::{self, GenesisConfigInfo, ValidatorVoteKeypairs},
+        hashed_transaction::HashedTransaction,
+        vote_sender_types::ReplayVoteSender,
+    },
+    safecoin_sdk::{pubkey::Pubkey, signature::Signer},
+    solana_vote_program::vote_transaction,
 };
-use safecoin_sdk::{pubkey::Pubkey, signature::Signer};
-use solana_vote_program::vote_transaction;
 
 pub fn setup_bank_and_vote_pubkeys(num_vote_accounts: usize, stake: u64) -> (Bank, Vec<Pubkey>) {
     // Create some voters at genesis
@@ -33,21 +35,21 @@ pub fn find_and_send_votes(
     vote_sender: Option<&ReplayVoteSender>,
 ) {
     let TransactionResults {
-        execution_results,
-        overwritten_vote_accounts,
-        ..
+        execution_results, ..
     } = tx_results;
     if let Some(vote_sender) = vote_sender {
-        for old_account in overwritten_vote_accounts {
-            assert!(execution_results[old_account.transaction_result_index]
-                .0
-                .is_ok());
-            let transaction = hashed_txs[old_account.transaction_index].transaction();
-            if let Some(parsed_vote) = vote_transaction::parse_vote_transaction(transaction) {
-                if parsed_vote.1.slots.last().is_some() {
-                    let _ = vote_sender.send(parsed_vote);
+        hashed_txs.iter().zip(execution_results.iter()).for_each(
+            |(hashed_tx, (result, _nonce_rollback))| {
+                let transaction = hashed_tx.transaction();
+                if hashed_tx.is_simple_vote_transaction() && result.is_ok() {
+                    if let Some(parsed_vote) = vote_transaction::parse_vote_transaction(transaction)
+                    {
+                        if parsed_vote.1.slots.last().is_some() {
+                            let _ = vote_sender.send(parsed_vote);
+                        }
+                    }
                 }
-            }
-        }
+            },
+        );
     }
 }
