@@ -1,6 +1,8 @@
-use safecoin_sdk::{clock::Slot, commitment_config::CommitmentLevel};
-use solana_vote_program::vote_state::MAX_LOCKOUT_HISTORY;
-use std::collections::HashMap;
+use {
+    safecoin_sdk::{clock::Slot, commitment_config::CommitmentLevel},
+    solana_vote_program::vote_state::MAX_LOCKOUT_HISTORY,
+    std::collections::HashMap,
+};
 
 pub const VOTE_THRESHOLD_SIZE: f64 = 1f64 / 60f64;    //to be raised on next upgrade
 pub const VOTE_THRESHOLD_SIZE_ORIG: f64 = 1f64 / 40f64;
@@ -47,14 +49,14 @@ pub struct BlockCommitmentCache {
     /// the struct to avoid the expense of recalculating on every call.
     commitment_slots: CommitmentSlots,
     /// Total stake active during the bank's epoch
-    threshold: f64,
+    total_stake: u64,
 }
 
 impl std::fmt::Debug for BlockCommitmentCache {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("BlockCommitmentCache")
             .field("block_commitment", &self.block_commitment)
-            .field("threshold", &self.threshold)
+            .field("total_stake", &self.total_stake)
             .field(
                 "bank",
                 &format_args!("Bank({{current_slot: {:?}}})", self.commitment_slots.slot),
@@ -67,23 +69,22 @@ impl std::fmt::Debug for BlockCommitmentCache {
 impl BlockCommitmentCache {
     pub fn new(
         block_commitment: HashMap<Slot, BlockCommitment>,
-        threshold: f64,
+        total_stake: u64,
         commitment_slots: CommitmentSlots,
     ) -> Self {
-            Self {
-                block_commitment,
-                commitment_slots,
-                threshold,
-            }
-       
+        Self {
+            block_commitment,
+            commitment_slots,
+            total_stake,
+        }
     }
 
     pub fn get_block_commitment(&self, slot: Slot) -> Option<&BlockCommitment> {
         self.block_commitment.get(&slot)
     }
 
-    pub fn threshold(&self) -> f64 {
-        self.threshold
+    pub fn total_stake(&self) -> u64 {
+        self.total_stake
     }
 
     pub fn slot(&self) -> Slot {
@@ -140,8 +141,9 @@ impl BlockCommitmentCache {
     pub fn calculate_highest_confirmed_slot(&self) -> Slot {
         self.highest_slot_with_confirmation_count(1)
     }
+
     pub fn get_confirmation_count(&self, slot: Slot) -> Option<usize> {
-        self.get_lockout_count(slot, self.threshold)
+        self.get_lockout_count(slot, VOTE_THRESHOLD_SIZE)
     }
 
     // Returns the lowest level at which at least `minimum_stake_percentage` of the total epoch
@@ -152,7 +154,7 @@ impl BlockCommitmentCache {
             let mut sum = 0;
             for (i, stake) in iterator {
                 sum += stake;
-                if sum as f64  > minimum_stake_percentage {
+                if (sum as f64 / self.total_stake as f64) > minimum_stake_percentage {
                     return i + 1;
                 }
             }
@@ -165,7 +167,7 @@ impl BlockCommitmentCache {
         block_commitment.insert(0, BlockCommitment::default());
         Self {
             block_commitment,
-            threshold: 0.6,
+            total_stake: 42,
             ..Self::default()
         }
     }
@@ -175,7 +177,7 @@ impl BlockCommitmentCache {
         block_commitment.insert(0, BlockCommitment::default());
         Self {
             block_commitment,
-            threshold: 0.6,
+            total_stake: 42,
             commitment_slots: CommitmentSlots {
                 slot,
                 root,
@@ -262,7 +264,7 @@ mod tests {
         block_commitment.entry(2).or_insert(cache2);
         let block_commitment_cache = BlockCommitmentCache {
             block_commitment,
-            threshold: 0.6,
+            total_stake: 50,
             ..BlockCommitmentCache::default()
         };
 
@@ -275,6 +277,7 @@ mod tests {
     #[test]
     fn test_highest_confirmed_slot() {
         let bank_slot_5 = 5;
+        let total_stake = 50;
 
         // Build cache with confirmation_count 2 given total_stake
         let mut cache0 = BlockCommitment::default();
@@ -297,7 +300,7 @@ mod tests {
         block_commitment.entry(3).or_insert_with(|| cache2.clone()); // Slot 3, conf 0
         let commitment_slots = CommitmentSlots::new_from_slot(bank_slot_5);
         let block_commitment_cache =
-            BlockCommitmentCache::new(block_commitment, 0.6, commitment_slots);
+            BlockCommitmentCache::new(block_commitment, total_stake, commitment_slots);
 
         assert_eq!(block_commitment_cache.calculate_highest_confirmed_slot(), 2);
 
@@ -307,7 +310,7 @@ mod tests {
         block_commitment.entry(2).or_insert_with(|| cache1.clone()); // Slot 2, conf 1
         block_commitment.entry(3).or_insert_with(|| cache2.clone()); // Slot 3, conf 0
         let block_commitment_cache =
-            BlockCommitmentCache::new(block_commitment, 0.66, commitment_slots);
+            BlockCommitmentCache::new(block_commitment, total_stake, commitment_slots);
 
         assert_eq!(block_commitment_cache.calculate_highest_confirmed_slot(), 2);
 
@@ -317,7 +320,7 @@ mod tests {
         block_commitment.entry(3).or_insert(cache1); // Slot 3, conf 1
         block_commitment.entry(5).or_insert_with(|| cache2.clone()); // Slot 5, conf 0
         let block_commitment_cache =
-            BlockCommitmentCache::new(block_commitment, 0.66, commitment_slots);
+            BlockCommitmentCache::new(block_commitment, total_stake, commitment_slots);
 
         assert_eq!(block_commitment_cache.calculate_highest_confirmed_slot(), 3);
 
@@ -327,7 +330,7 @@ mod tests {
         block_commitment.entry(2).or_insert_with(|| cache2.clone()); // Slot 2, conf 0
         block_commitment.entry(3).or_insert_with(|| cache2.clone()); // Slot 3, conf 0
         let block_commitment_cache =
-            BlockCommitmentCache::new(block_commitment, 0.6, commitment_slots);
+            BlockCommitmentCache::new(block_commitment, total_stake, commitment_slots);
 
         assert_eq!(block_commitment_cache.calculate_highest_confirmed_slot(), 1);
 
@@ -337,7 +340,7 @@ mod tests {
         block_commitment.entry(2).or_insert_with(|| cache2.clone()); // Slot 2, conf 0
         block_commitment.entry(3).or_insert(cache2); // Slot 3, conf 0
         let block_commitment_cache =
-            BlockCommitmentCache::new(block_commitment, 0.6, commitment_slots);
+            BlockCommitmentCache::new(block_commitment, total_stake, commitment_slots);
 
         assert_eq!(block_commitment_cache.calculate_highest_confirmed_slot(), 0);
     }
