@@ -1,8 +1,8 @@
 // Partial SPL Token v2.0.x declarations inlined to avoid an external dependency on the safe-token crate
-solana_sdk::declare_id!("HMGr16f8Ct1Zeb9TGPypt9rPgzCkmhCQB8Not8vwiPW1");
+solana_sdk::declare_id!("ToKLx75MGim1d1jRusuVX8xvdvvbSDESVaNXpRA9PHN");
 
 pub(crate) mod new_token_program {
-    solana_sdk::declare_id!("t31zsgDmRntje65uXV3LrnWaJtJJpMd4LyJxq2R2VrU");
+    solana_sdk::declare_id!("nTokHfnBtpt4V6xiEbBSduiGCrQ6wSF3rxC8WeWAQ9F");
 }
 
 /*
@@ -31,7 +31,7 @@ pub mod state {
 }
 
 pub mod native_mint {
-    solana_sdk::declare_id!("Safe111111111111111111111111111111111111112");
+    solana_sdk::declare_id!("Safe111111111111111111111111111111111111111");
 
     /*
         Mint {
