@@ -1,3 +1,5 @@
+#[cfg(RUSTC_WITH_SPECIALIZATION)]
+use safecoin_frozen_abi::abi_example::IgnoreAsHelper;
 use {
     crate::{
         accounts::Accounts,
@@ -5,6 +7,7 @@ use {
             AccountShrinkThreshold, AccountStorageEntry, AccountsDb, AppendVecId, BankHashInfo,
         },
         accounts_index::AccountSecondaryIndexes,
+        accounts_update_notifier_interface::AccountsUpdateNotifier,
         ancestors::Ancestors,
         append_vec::{AppendVec, StoredMetaWriteVersion},
         bank::{Bank, BankFieldsToDeserialize, BankRc, Builtins},
@@ -16,17 +19,16 @@ use {
         serde_snapshot::future::SerializableStorage,
         stakes::Stakes,
     },
-    bincode,
-    bincode::{config::Options, Error},
+    bincode::{self, config::Options, Error},
     log::*,
     rayon::prelude::*,
     serde::{de::DeserializeOwned, Deserialize, Serialize},
+    safecoin_measure::measure::Measure,
     safecoin_sdk::{
         clock::{Epoch, Slot, UnixTimestamp},
         epoch_schedule::EpochSchedule,
         fee_calculator::{FeeCalculator, FeeRateGovernor},
-        genesis_config::ClusterType,
-        genesis_config::GenesisConfig,
+        genesis_config::{ClusterType, GenesisConfig},
         hard_forks::HardForks,
         hash::Hash,
         inflation::Inflation,
@@ -41,23 +43,18 @@ use {
     },
 };
 
-#[cfg(RUSTC_WITH_SPECIALIZATION)]
-use safecoin_frozen_abi::abi_example::IgnoreAsHelper;
-
 mod common;
 mod future;
 mod tests;
 mod utils;
 
-use future::Context as TypeContextFuture;
-#[allow(unused_imports)]
-use utils::{serialize_iter_as_map, serialize_iter_as_seq, serialize_iter_as_tuple};
-
 // a number of test cases in accounts_db use this
 #[cfg(test)]
 pub(crate) use self::tests::reconstruct_accounts_db_via_serialization;
-
 pub(crate) use crate::accounts_db::{SnapshotStorage, SnapshotStorages};
+use future::Context as TypeContextFuture;
+#[allow(unused_imports)]
+use utils::{serialize_iter_as_map, serialize_iter_as_seq, serialize_iter_as_tuple};
 
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub(crate) enum SerdeStyle {
@@ -140,6 +137,7 @@ pub(crate) fn bank_from_stream<R>(
     caching_enabled: bool,
     limit_load_slot_count_from_snapshot: Option<usize>,
     shrink_ratio: AccountShrinkThreshold,
+    accounts_update_notifier: Option<AccountsUpdateNotifier>,
 ) -> std::result::Result<Bank, Error>
 where
     R: Read,
@@ -161,6 +159,7 @@ where
                 caching_enabled,
                 limit_load_slot_count_from_snapshot,
                 shrink_ratio,
+                accounts_update_notifier,
             )?;
             Ok(bank)
         }};
@@ -252,6 +251,7 @@ fn reconstruct_bank_from_fields<E>(
     caching_enabled: bool,
     limit_load_slot_count_from_snapshot: Option<usize>,
     shrink_ratio: AccountShrinkThreshold,
+    accounts_update_notifier: Option<AccountsUpdateNotifier>,
 ) -> Result<Bank, Error>
 where
     E: SerializableStorage + std::marker::Sync,
@@ -265,6 +265,7 @@ where
         caching_enabled,
         limit_load_slot_count_from_snapshot,
         shrink_ratio,
+        accounts_update_notifier,
     )?;
     accounts_db.freeze_accounts(
         &Ancestors::from(&bank_fields.ancestors),
@@ -314,6 +315,7 @@ fn reconstruct_accountsdb_from_fields<E>(
     caching_enabled: bool,
     limit_load_slot_count_from_snapshot: Option<usize>,
     shrink_ratio: AccountShrinkThreshold,
+    accounts_update_notifier: Option<AccountsUpdateNotifier>,
 ) -> Result<AccountsDb, Error>
 where
     E: SerializableStorage + std::marker::Sync,
@@ -324,6 +326,7 @@ where
         account_indexes,
         caching_enabled,
         shrink_ratio,
+        accounts_update_notifier,
     );
     let AccountsDbFields(storage, version, slot, bank_hash_info) = accounts_db_fields;
 
@@ -388,6 +391,15 @@ where
         );
     }
 
+    let mut measure_notify = Measure::start("accounts_notify");
+    accounts_db.notify_account_restore_from_snapshot();
+    measure_notify.stop();
+
+    datapoint_info!(
+        "reconstruct_accountsdb_from_fields()",
+        ("accountsdb-notify-at-start-us", measure_notify.as_us(), i64),
+    );
+
     if max_id > AppendVecId::MAX / 2 {
         panic!("Storage id {} larger than allowed max", max_id);
     }
