@@ -1,15 +1,20 @@
-use serde::de::{Deserialize, Deserializer};
-use serde::ser::{Serialize, Serializer};
-use safecoin_sdk::{
-    account::Account, account::AccountSharedData, instruction::InstructionError, pubkey::Pubkey,
-};
-use solana_vote_program::vote_state::VoteState;
-use std::{
-    cmp::Ordering,
-    collections::{hash_map::Entry, HashMap},
-    iter::FromIterator,
-    ops::Deref,
-    sync::{Arc, Once, RwLock, RwLockReadGuard},
+use {
+    serde::{
+        de::{Deserialize, Deserializer},
+        ser::{Serialize, Serializer},
+    },
+    safecoin_sdk::{
+        account::{Account, AccountSharedData},
+        instruction::InstructionError,
+        pubkey::Pubkey,
+    },
+    solana_vote_program::vote_state::VoteState,
+    std::{
+        cmp::Ordering,
+        collections::{hash_map::Entry, HashMap},
+        iter::FromIterator,
+        sync::{Arc, Once, RwLock, RwLockReadGuard},
+    },
 };
 
 // The value here does not matter. It will be overwritten
@@ -18,20 +23,22 @@ const INVALID_VOTE_STATE: Result<VoteState, InstructionError> =
     Err(InstructionError::InvalidAccountData);
 
 #[derive(Clone, Debug, Default, PartialEq, AbiExample)]
-pub struct ArcVoteAccount(Arc<VoteAccount>);
+pub struct VoteAccount(Arc<VoteAccountInner>);
 
 #[derive(Debug, AbiExample)]
-pub struct VoteAccount {
+struct VoteAccountInner {
     account: Account,
     vote_state: RwLock<Result<VoteState, InstructionError>>,
     vote_state_once: Once,
 }
 
-pub type VoteAccountsHashMap = HashMap<Pubkey, (/*stake:*/ u64, ArcVoteAccount)>;
+pub type VoteAccountsHashMap = HashMap<Pubkey, (/*stake:*/ u64, VoteAccount)>;
 
 #[derive(Debug, AbiExample)]
 pub struct VoteAccounts {
-    vote_accounts: VoteAccountsHashMap,
+    vote_accounts: Arc<VoteAccountsHashMap>,
+    // Inner Arc is meant to implement copy-on-write semantics as opposed to
+    // sharing mutations (hence RwLock<Arc<...>> instead of Arc<RwLock<...>>).
     staked_nodes: RwLock<
         HashMap<
             Pubkey, // VoteAccount.vote_state.node_pubkey.
@@ -42,19 +49,17 @@ pub struct VoteAccounts {
 }
 
 impl VoteAccount {
-    pub fn account(&self) -> &Account {
-        &self.account
-    }
-
-    pub fn lamports(&self) -> u64 {
-        self.account.lamports
+    pub(crate) fn lamports(&self) -> u64 {
+        self.0.account.lamports
     }
 
     pub fn vote_state(&self) -> RwLockReadGuard<Result<VoteState, InstructionError>> {
-        self.vote_state_once.call_once(|| {
-            *self.vote_state.write().unwrap() = VoteState::deserialize(&self.account.data);
+        let inner = &self.0;
+        inner.vote_state_once.call_once(|| {
+            let vote_state = VoteState::deserialize(&inner.account.data);
+            *inner.vote_state.write().unwrap() = vote_state;
         });
-        self.vote_state.read().unwrap()
+        inner.vote_state.read().unwrap()
     }
 
     /// VoteState.node_pubkey of this vote-account.
@@ -82,37 +87,43 @@ impl VoteAccounts {
         self.staked_nodes.read().unwrap().clone()
     }
 
-    pub fn iter(&self) -> impl Iterator<Item = (&Pubkey, &(u64, ArcVoteAccount))> {
+    pub fn get(&self, pubkey: &Pubkey) -> Option<&(/*stake:*/ u64, VoteAccount)> {
+        self.vote_accounts.get(pubkey)
+    }
+
+    pub(crate) fn iter(&self) -> impl Iterator<Item = (&Pubkey, &(u64, VoteAccount))> {
         self.vote_accounts.iter()
     }
 
-    pub fn insert(&mut self, pubkey: Pubkey, (stake, vote_account): (u64, ArcVoteAccount)) {
+    pub(crate) fn insert(&mut self, pubkey: Pubkey, (stake, vote_account): (u64, VoteAccount)) {
         self.add_node_stake(stake, &vote_account);
-        if let Some((stake, vote_account)) =
-            self.vote_accounts.insert(pubkey, (stake, vote_account))
-        {
+        let vote_accounts = Arc::make_mut(&mut self.vote_accounts);
+        if let Some((stake, vote_account)) = vote_accounts.insert(pubkey, (stake, vote_account)) {
             self.sub_node_stake(stake, &vote_account);
         }
     }
 
-    pub fn remove(&mut self, pubkey: &Pubkey) -> Option<(u64, ArcVoteAccount)> {
-        let value = self.vote_accounts.remove(pubkey);
-        if let Some((stake, ref vote_account)) = value {
+    pub(crate) fn remove(&mut self, pubkey: &Pubkey) -> Option<(u64, VoteAccount)> {
+        let vote_accounts = Arc::make_mut(&mut self.vote_accounts);
+        let entry = vote_accounts.remove(pubkey);
+        if let Some((stake, ref vote_account)) = entry {
             self.sub_node_stake(stake, vote_account);
         }
-        value
+        entry
     }
 
-    pub fn add_stake(&mut self, pubkey: &Pubkey, delta: u64) {
-        if let Some((stake, vote_account)) = self.vote_accounts.get_mut(pubkey) {
+    pub(crate) fn add_stake(&mut self, pubkey: &Pubkey, delta: u64) {
+        let vote_accounts = Arc::make_mut(&mut self.vote_accounts);
+        if let Some((stake, vote_account)) = vote_accounts.get_mut(pubkey) {
             *stake += delta;
             let vote_account = vote_account.clone();
             self.add_node_stake(delta, &vote_account);
         }
     }
 
-    pub fn sub_stake(&mut self, pubkey: &Pubkey, delta: u64) {
-        if let Some((stake, vote_account)) = self.vote_accounts.get_mut(pubkey) {
+    pub(crate) fn sub_stake(&mut self, pubkey: &Pubkey, delta: u64) {
+        let vote_accounts = Arc::make_mut(&mut self.vote_accounts);
+        if let Some((stake, vote_account)) = vote_accounts.get_mut(pubkey) {
             *stake = stake
                 .checked_sub(delta)
                 .expect("subtraction value exceeds account's stake");
@@ -121,7 +132,7 @@ impl VoteAccounts {
         }
     }
 
-    fn add_node_stake(&mut self, stake: u64, vote_account: &ArcVoteAccount) {
+    fn add_node_stake(&mut self, stake: u64, vote_account: &VoteAccount) {
         if stake != 0 && self.staked_nodes_once.is_completed() {
             if let Some(node_pubkey) = vote_account.node_pubkey() {
                 self.staked_nodes
@@ -134,7 +145,7 @@ impl VoteAccounts {
         }
     }
 
-    fn sub_node_stake(&mut self, stake: u64, vote_account: &ArcVoteAccount) {
+    fn sub_node_stake(&mut self, stake: u64, vote_account: &VoteAccount) {
         if stake != 0 && self.staked_nodes_once.is_completed() {
             if let Some(node_pubkey) = vote_account.node_pubkey() {
                 match self.staked_nodes.write().unwrap().entry(node_pubkey) {
@@ -152,24 +163,16 @@ impl VoteAccounts {
     }
 }
 
-impl Deref for ArcVoteAccount {
-    type Target = VoteAccount;
-
-    fn deref(&self) -> &Self::Target {
-        self.0.deref()
-    }
-}
-
-impl Serialize for ArcVoteAccount {
+impl Serialize for VoteAccount {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        self.account.serialize(serializer)
+        self.0.account.serialize(serializer)
     }
 }
 
-impl<'de> Deserialize<'de> for ArcVoteAccount {
+impl<'de> Deserialize<'de> for VoteAccount {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
     where
         D: Deserializer<'de>,
@@ -179,34 +182,25 @@ impl<'de> Deserialize<'de> for ArcVoteAccount {
     }
 }
 
-impl From<AccountSharedData> for ArcVoteAccount {
-    fn from(account: AccountSharedData) -> Self {
-        Self(Arc::new(VoteAccount::from(account)))
-    }
-}
-impl From<Account> for ArcVoteAccount {
-    fn from(account: Account) -> Self {
-        Self(Arc::new(VoteAccount::from(account)))
-    }
-}
-
-impl AsRef<VoteAccount> for ArcVoteAccount {
-    fn as_ref(&self) -> &VoteAccount {
-        &self.0
-    }
-}
-
 impl From<AccountSharedData> for VoteAccount {
     fn from(account: AccountSharedData) -> Self {
-        Self {
-            account: Account::from(account),
-            vote_state: RwLock::new(INVALID_VOTE_STATE),
-            vote_state_once: Once::new(),
-        }
+        Self(Arc::new(VoteAccountInner::from(account)))
     }
 }
 
 impl From<Account> for VoteAccount {
+    fn from(account: Account) -> Self {
+        Self(Arc::new(VoteAccountInner::from(account)))
+    }
+}
+
+impl From<AccountSharedData> for VoteAccountInner {
+    fn from(account: AccountSharedData) -> Self {
+        Self::from(Account::from(account))
+    }
+}
+
+impl From<Account> for VoteAccountInner {
     fn from(account: Account) -> Self {
         Self {
             account,
@@ -216,7 +210,7 @@ impl From<Account> for VoteAccount {
     }
 }
 
-impl Default for VoteAccount {
+impl Default for VoteAccountInner {
     fn default() -> Self {
         Self {
             account: Account::default(),
@@ -226,7 +220,7 @@ impl Default for VoteAccount {
     }
 }
 
-impl PartialEq<VoteAccount> for VoteAccount {
+impl PartialEq<VoteAccountInner> for VoteAccountInner {
     fn eq(&self, other: &Self) -> bool {
         self.account == other.account
     }
@@ -235,7 +229,7 @@ impl PartialEq<VoteAccount> for VoteAccount {
 impl Default for VoteAccounts {
     fn default() -> Self {
         Self {
-            vote_accounts: HashMap::default(),
+            vote_accounts: Arc::default(),
             staked_nodes: RwLock::default(),
             staked_nodes_once: Once::new(),
         }
@@ -269,8 +263,8 @@ impl PartialEq<VoteAccounts> for VoteAccounts {
     }
 }
 
-impl From<VoteAccountsHashMap> for VoteAccounts {
-    fn from(vote_accounts: VoteAccountsHashMap) -> Self {
+impl From<Arc<VoteAccountsHashMap>> for VoteAccounts {
+    fn from(vote_accounts: Arc<VoteAccountsHashMap>) -> Self {
         Self {
             vote_accounts,
             staked_nodes: RwLock::default(),
@@ -285,12 +279,18 @@ impl AsRef<VoteAccountsHashMap> for VoteAccounts {
     }
 }
 
-impl FromIterator<(Pubkey, (u64 /*stake*/, ArcVoteAccount))> for VoteAccounts {
+impl From<&VoteAccounts> for Arc<VoteAccountsHashMap> {
+    fn from(vote_accounts: &VoteAccounts) -> Self {
+        Arc::clone(&vote_accounts.vote_accounts)
+    }
+}
+
+impl FromIterator<(Pubkey, (/*stake:*/ u64, VoteAccount))> for VoteAccounts {
     fn from_iter<I>(iter: I) -> Self
     where
-        I: IntoIterator<Item = (Pubkey, (u64, ArcVoteAccount))>,
+        I: IntoIterator<Item = (Pubkey, (u64, VoteAccount))>,
     {
-        Self::from(HashMap::from_iter(iter))
+        Self::from(Arc::new(HashMap::from_iter(iter)))
     }
 }
 
@@ -309,18 +309,20 @@ impl<'de> Deserialize<'de> for VoteAccounts {
         D: Deserializer<'de>,
     {
         let vote_accounts = VoteAccountsHashMap::deserialize(deserializer)?;
-        Ok(Self::from(vote_accounts))
+        Ok(Self::from(Arc::new(vote_accounts)))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use bincode::Options;
-    use rand::Rng;
-    use safecoin_sdk::{pubkey::Pubkey, sysvar::clock::Clock};
-    use solana_vote_program::vote_state::{VoteInit, VoteStateVersions};
-    use std::iter::repeat_with;
+    use {
+        super::*,
+        bincode::Options,
+        rand::Rng,
+        safecoin_sdk::{pubkey::Pubkey, sysvar::clock::Clock},
+        solana_vote_program::vote_state::{VoteInit, VoteStateVersions},
+        std::iter::repeat_with,
+    };
 
     fn new_rand_vote_account<R: Rng>(
         rng: &mut R,
@@ -352,19 +354,19 @@ mod tests {
     fn new_rand_vote_accounts<R: Rng>(
         rng: &mut R,
         num_nodes: usize,
-    ) -> impl Iterator<Item = (Pubkey, (u64 /*stake*/, ArcVoteAccount))> + '_ {
+    ) -> impl Iterator<Item = (Pubkey, (/*stake:*/ u64, VoteAccount))> + '_ {
         let nodes: Vec<_> = repeat_with(Pubkey::new_unique).take(num_nodes).collect();
         repeat_with(move || {
             let node = nodes[rng.gen_range(0, nodes.len())];
             let (account, _) = new_rand_vote_account(rng, Some(node));
             let stake = rng.gen_range(0, 997);
-            (Pubkey::new_unique(), (stake, ArcVoteAccount::from(account)))
+            (Pubkey::new_unique(), (stake, VoteAccount::from(account)))
         })
     }
 
     fn staked_nodes<'a, I>(vote_accounts: I) -> HashMap<Pubkey, u64>
     where
-        I: IntoIterator<Item = &'a (Pubkey, (u64, ArcVoteAccount))>,
+        I: IntoIterator<Item = &'a (Pubkey, (u64, VoteAccount))>,
     {
         let mut staked_nodes = HashMap::new();
         for (_, (stake, vote_account)) in vote_accounts
@@ -386,7 +388,7 @@ mod tests {
         let mut rng = rand::thread_rng();
         let (account, vote_state) = new_rand_vote_account(&mut rng, None);
         let lamports = account.lamports;
-        let vote_account = ArcVoteAccount::from(account);
+        let vote_account = VoteAccount::from(account);
         assert_eq!(lamports, vote_account.lamports());
         assert_eq!(vote_state, *vote_account.vote_state().as_ref().unwrap());
         // 2nd call to .vote_state() should return the cached value.
@@ -397,9 +399,9 @@ mod tests {
     fn test_vote_account_serialize() {
         let mut rng = rand::thread_rng();
         let (account, vote_state) = new_rand_vote_account(&mut rng, None);
-        let vote_account = ArcVoteAccount::from(account.clone());
+        let vote_account = VoteAccount::from(account.clone());
         assert_eq!(vote_state, *vote_account.vote_state().as_ref().unwrap());
-        // Assert than ArcVoteAccount has the same wire format as Account.
+        // Assert than VoteAccount has the same wire format as Account.
         assert_eq!(
             bincode::serialize(&account).unwrap(),
             bincode::serialize(&vote_account).unwrap()
@@ -411,9 +413,9 @@ mod tests {
         let mut rng = rand::thread_rng();
         let (account, vote_state) = new_rand_vote_account(&mut rng, None);
         let data = bincode::serialize(&account).unwrap();
-        let vote_account = ArcVoteAccount::from(account);
+        let vote_account = VoteAccount::from(account);
         assert_eq!(vote_state, *vote_account.vote_state().as_ref().unwrap());
-        let other_vote_account: ArcVoteAccount = bincode::deserialize(&data).unwrap();
+        let other_vote_account: VoteAccount = bincode::deserialize(&data).unwrap();
         assert_eq!(vote_account, other_vote_account);
         assert_eq!(
             vote_state,
@@ -425,11 +427,11 @@ mod tests {
     fn test_vote_account_round_trip() {
         let mut rng = rand::thread_rng();
         let (account, vote_state) = new_rand_vote_account(&mut rng, None);
-        let vote_account = ArcVoteAccount::from(account);
+        let vote_account = VoteAccount::from(account);
         assert_eq!(vote_state, *vote_account.vote_state().as_ref().unwrap());
         let data = bincode::serialize(&vote_account).unwrap();
-        let other_vote_account: ArcVoteAccount = bincode::deserialize(&data).unwrap();
-        // Assert that serialize->deserialized returns the same ArcVoteAccount.
+        let other_vote_account: VoteAccount = bincode::deserialize(&data).unwrap();
+        // Assert that serialize->deserialized returns the same VoteAccount.
         assert_eq!(vote_account, other_vote_account);
         assert_eq!(
             vote_state,
@@ -440,9 +442,9 @@ mod tests {
     #[test]
     fn test_vote_accounts_serialize() {
         let mut rng = rand::thread_rng();
-        let vote_accounts_hash_map: HashMap<Pubkey, (u64, ArcVoteAccount)> =
+        let vote_accounts_hash_map: HashMap<Pubkey, (u64, VoteAccount)> =
             new_rand_vote_accounts(&mut rng, 64).take(1024).collect();
-        let vote_accounts = VoteAccounts::from(vote_accounts_hash_map.clone());
+        let vote_accounts = VoteAccounts::from(Arc::new(vote_accounts_hash_map.clone()));
         assert!(vote_accounts.staked_nodes().len() > 32);
         assert_eq!(
             bincode::serialize(&vote_accounts).unwrap(),
@@ -459,17 +461,17 @@ mod tests {
     #[test]
     fn test_vote_accounts_deserialize() {
         let mut rng = rand::thread_rng();
-        let vote_accounts_hash_map: HashMap<Pubkey, (u64, ArcVoteAccount)> =
+        let vote_accounts_hash_map: HashMap<Pubkey, (u64, VoteAccount)> =
             new_rand_vote_accounts(&mut rng, 64).take(1024).collect();
         let data = bincode::serialize(&vote_accounts_hash_map).unwrap();
         let vote_accounts: VoteAccounts = bincode::deserialize(&data).unwrap();
         assert!(vote_accounts.staked_nodes().len() > 32);
-        assert_eq!(vote_accounts.vote_accounts, vote_accounts_hash_map);
+        assert_eq!(*vote_accounts.vote_accounts, vote_accounts_hash_map);
         let data = bincode::options()
             .serialize(&vote_accounts_hash_map)
             .unwrap();
         let vote_accounts: VoteAccounts = bincode::options().deserialize(&data).unwrap();
-        assert_eq!(vote_accounts.vote_accounts, vote_accounts_hash_map);
+        assert_eq!(*vote_accounts.vote_accounts, vote_accounts_hash_map);
     }
 
     #[test]
@@ -522,4 +524,70 @@ mod tests {
         }
         assert!(vote_accounts.staked_nodes.read().unwrap().is_empty());
     }
+
+    // Asserts that returned staked-nodes are copy-on-write references.
+    #[test]
+    fn test_staked_nodes_cow() {
+        let mut rng = rand::thread_rng();
+        let mut accounts = new_rand_vote_accounts(&mut rng, 64);
+        // Add vote accounts.
+        let mut vote_accounts = VoteAccounts::default();
+        for (pubkey, (stake, vote_account)) in (&mut accounts).take(1024) {
+            vote_accounts.insert(pubkey, (stake, vote_account));
+        }
+        let staked_nodes = vote_accounts.staked_nodes();
+        let (pubkey, (more_stake, vote_account)) =
+            accounts.find(|(_, (stake, _))| *stake != 0).unwrap();
+        let node_pubkey = vote_account.node_pubkey().unwrap();
+        vote_accounts.insert(pubkey, (more_stake, vote_account));
+        assert_ne!(staked_nodes, vote_accounts.staked_nodes());
+        assert_eq!(
+            vote_accounts.staked_nodes()[&node_pubkey],
+            more_stake + staked_nodes.get(&node_pubkey).copied().unwrap_or_default()
+        );
+        for (pubkey, stake) in vote_accounts.staked_nodes().iter() {
+            if *pubkey != node_pubkey {
+                assert_eq!(*stake, staked_nodes[pubkey]);
+            } else {
+                assert_eq!(
+                    *stake,
+                    more_stake + staked_nodes.get(pubkey).copied().unwrap_or_default()
+                );
+            }
+        }
+    }
+
+    // Asserts that returned vote-accounts are copy-on-write references.
+    #[test]
+    fn test_vote_accounts_cow() {
+        let mut rng = rand::thread_rng();
+        let mut accounts = new_rand_vote_accounts(&mut rng, 64);
+        // Add vote accounts.
+        let mut vote_accounts = VoteAccounts::default();
+        for (pubkey, (stake, vote_account)) in (&mut accounts).take(1024) {
+            vote_accounts.insert(pubkey, (stake, vote_account));
+        }
+        let vote_accounts_hashmap = Arc::<VoteAccountsHashMap>::from(&vote_accounts);
+        assert_eq!(vote_accounts_hashmap, vote_accounts.vote_accounts);
+        assert!(Arc::ptr_eq(
+            &vote_accounts_hashmap,
+            &vote_accounts.vote_accounts
+        ));
+        let (pubkey, (more_stake, vote_account)) =
+            accounts.find(|(_, (stake, _))| *stake != 0).unwrap();
+        vote_accounts.insert(pubkey, (more_stake, vote_account.clone()));
+        assert!(!Arc::ptr_eq(
+            &vote_accounts_hashmap,
+            &vote_accounts.vote_accounts
+        ));
+        assert_ne!(vote_accounts_hashmap, vote_accounts.vote_accounts);
+        let other = (more_stake, vote_account);
+        for (pk, value) in vote_accounts.iter() {
+            if *pk != pubkey {
+                assert_eq!(value, &vote_accounts_hashmap[pk]);
+            } else {
+                assert_eq!(value, &other);
+            }
+        }
+    }
 }
