@@ -1,6 +1,8 @@
-use solana_runtime::bank::Bank;
-use safecoin_sdk::{genesis_config::create_genesis_config, hash::hash};
-use std::{sync::Arc, thread::Builder};
+use {
+    solana_runtime::bank::Bank,
+    safecoin_sdk::{genesis_config::create_genesis_config, hash::hash},
+    std::{sync::Arc, thread::Builder},
+};
 
 #[test]
 fn test_race_register_tick_freeze() {
